require "Window"
require "Unit"
require "GameLib"
require "Apollo"
 
 if not STARTUP_FUNC then
	STARTUP_LIST = {}
	function STARTUP_FUNC(m)
		local t = GameLib.GetTickCount()
		table.insert(STARTUP_LIST, string.format("%s [%.4f]",m, t))
	end
end

STARTUP_FUNC("TargetArrow:Top of TargetArrow.lua")

local TargetArrow = {} 

TargetArrow.Settings = {
	nScale = 1.0, nOffset = -100,
	bShowFriendly = true, bShowHostile = true, bShowNeutral = true,
	bShowPlayers = true, bShowNPC = true, bShowOthers = true,
	bShowInCombat = true, bShowOutCombat = true
}
 
TargetArrow.currentUnit = nil
TargetArrow.playerCombat = false
 
function TargetArrow:new(o)
	STARTUP_FUNC("TargetArrow:New")
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
	return o
end

function TargetArrow:Init()
	STARTUP_FUNC("TargetArrow:Init")
	local bHasConfigureFunction = true
	local strConfigureButtonText = "TargetArrow"
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 
function TargetArrow:OnConfigure()
	if self.xmlCfg and self.xmlCfg:IsLoaded() then
		self.wndCfg:Show(true, true)
	end
end
 
function TargetArrow:OnLoad()
	STARTUP_FUNC("TargetArrow:OnLoad")
	Apollo.LoadSprites("sprTargetArrow.xml", "TargetArrow")
	self.xmlDoc = XmlDoc.CreateFromFile("TargetArrow.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	self.xmlCfg = XmlDoc.CreateFromFile("TargetArrow_Config.xml")
	self.xmlCfg:RegisterCallback("OnDocLoadedCFG", self)
end

function TargetArrow:OnDocLoadedCFG()
	STARTUP_FUNC("TargetArrow:OnDocLoadedCFG")
	if self.xmlCfg and self.xmlCfg:IsLoaded() then
		self.wndCfg = Apollo.LoadForm(self.xmlCfg, "TargetArrowConfig", nil, self)
		if self.wndCfg == nil then
			Apollo.AddAddonErrorText(self, "Could not load the configuration window for some reason.")
			return
		end
		self.wndCfg:Show(false, true)
		
		TargetArrow.lblScale = self.wndCfg:FindChild("ScaleSlider")
		TargetArrow.sldScale = TargetArrow.lblScale:FindChild("Slider")
		TargetArrow.lblOffset = self.wndCfg:FindChild("OffsetSlider")
		TargetArrow.sldOffset = TargetArrow.lblOffset:FindChild("Slider")
		TargetArrow.bFriendly = self.wndCfg:FindChild("bFriendly")
		TargetArrow.bNeutral = self.wndCfg:FindChild("bNeutral")
		TargetArrow.bHostile = self.wndCfg:FindChild("bHostile")
		TargetArrow.bPlayers = self.wndCfg:FindChild("bPlayers")
		TargetArrow.bNPC = self.wndCfg:FindChild("bNPC")
		TargetArrow.bOther = self.wndCfg:FindChild("bOther")		
		TargetArrow.bInCombat = self.wndCfg:FindChild("bInCombat")		
		TargetArrow.bOutCombat = self.wndCfg:FindChild("bOutCombat")		

		TargetArrow.sldScale:SetValue(TargetArrow.Settings.nScale)
		TargetArrow.lblScale:SetText(string.format("%.2f", TargetArrow.Settings.nScale))
		TargetArrow.sldOffset:SetValue(TargetArrow.Settings.nOffset)
		TargetArrow.lblOffset:SetText(string.format("%d", TargetArrow.Settings.nOffset))
		
		TargetArrow.bFriendly:SetCheck(TargetArrow.Settings.bShowFriendly)
		TargetArrow.bNeutral:SetCheck(TargetArrow.Settings.bShowNeutral)
		TargetArrow.bHostile:SetCheck(TargetArrow.Settings.bShowHostile)
		TargetArrow.bPlayers:SetCheck(TargetArrow.Settings.bShowPlayers)
		TargetArrow.bNPC:SetCheck(TargetArrow.Settings.bShowNPC)
		TargetArrow.bOther:SetCheck(TargetArrow.Settings.bShowOthers)
		TargetArrow.bInCombat:SetCheck(TargetArrow.Settings.bShowInCombat)
		TargetArrow.bOutCombat:SetCheck(TargetArrow.Settings.bShowOutCombat)
	end
end

function TargetArrow:OnDocLoaded()
	STARTUP_FUNC("TargetArrow:OnDocLoaded")
	if self.xmlDoc and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "Arrow", "InWorldHudStratum", self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		Apollo.RegisterEventHandler("TargetUnitChanged", 	"OnTargetUnitChanged", self)
		Apollo.RegisterEventHandler("UnitEnteredCombat", 	"OnUnitEnteredCombat", self)
	    self.wndMain:Show(false, true)
		self:HandleSettings()
	end
end

function TargetArrow:OnTargetUnitChanged(unitID)
	TargetArrow.currentUnit = unitID
	local sTargetArrow = false
	if unitID then
		local nDisposition = GameLib.GetPlayerUnit():GetDispositionTo(unitID)
		
		if nDisposition == Unit.CodeEnumDisposition.Hostile then
			sTargetArrow = TargetArrow.Settings.bShowHostile
		elseif nDisposition == Unit.CodeEnumDisposition.Neutral then
			sTargetArrow = TargetArrow.Settings.bShowNeutral
		elseif nDisposition == Unit.CodeEnumDisposition.Friendly then
			sTargetArrow = TargetArrow.Settings.bShowFriendly
		end
			
		if sTargetArrow then
			local nType = unitID:GetType()
			if nType == "Player" then 
				if not TargetArrow.Settings.bShowPlayers then
					sTargetArrow = false
				end
			elseif nType == "NonPlayer" then
				if not TargetArrow.Settings.bShowNPC then
					sTargetArrow = false
				end
			else
				if not TargetArrow.Settings.bShowOthers then
					sTargetArrow = false
				end
			end
		end
		
		if sTargetArrow then
			if TargetArrow.playerCombat then
				sTargetArrow = TargetArrow.Settings.bShowInCombat
			else
				sTargetArrow = TargetArrow.Settings.bShowOutCombat
			end
		end
	end
	
	if sTargetArrow then
		self.wndMain:Show(true, true)
		self.wndMain:SetData(unitID)
		self.wndMain:SetUnit(unitID)
	else
		self.wndMain:Show(false, true)
	end
end

function TargetArrow:HandleSettings()
	local sprW = math.floor(TargetArrow.Settings.nScale * 75)
	local sprH = math.floor(TargetArrow.Settings.nScale * 150)
	
	local nOL, nOT, nOR, nOB
	
	nOL = math.floor(sprW/2)
	nOR = sprW-nOL
	nOT = 0 + TargetArrow.Settings.nOffset
	nOB = sprH + TargetArrow.Settings.nOffset

	self.wndMain:SetAnchorOffsets(-nOL, nOT, nOR, nOB)
end

function TargetArrow:OnSave(eType)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		return TargetArrow.Settings
	end
end

function TargetArrow:OnRestore(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		for k,v in pairs(tSavedData) do
			TargetArrow.Settings[k] = v
		end
	end
end

function TargetArrow:OnUnitEnteredCombat(unitID, inCombat)
	if unitID == GameLib:GetPlayerUnit() then
		TargetArrow.playerCombat = inCombat
		self:OnTargetUnitChanged(TargetArrow.currentUnit)
	end
end

function TargetArrow:OnSliderBarChanged()
	local uiValue
	uiValue = math.floor(TargetArrow.sldScale:GetValue()*20)/20
	TargetArrow.lblScale:SetText(string.format("%.2f", uiValue))
	TargetArrow.Settings.nScale = uiValue
	uiValue = math.floor(TargetArrow.sldOffset:GetValue()/5)*5
	TargetArrow.lblOffset:SetText(string.format("%d", uiValue))
	TargetArrow.Settings.nOffset = uiValue
	self:HandleSettings()
end

function TargetArrow:OnButtonChange()
	TargetArrow.Settings.bShowFriendly = TargetArrow.bFriendly:IsChecked()
	TargetArrow.Settings.bShowNeutral = TargetArrow.bNeutral:IsChecked()
	TargetArrow.Settings.bShowHostile = TargetArrow.bHostile:IsChecked()
	TargetArrow.Settings.bShowPlayers = TargetArrow.bPlayers:IsChecked()
	TargetArrow.Settings.bShowNPC = TargetArrow.bNPC:IsChecked()
	TargetArrow.Settings.bShowOthers = TargetArrow.bOther:IsChecked()
	TargetArrow.Settings.bShowInCombat = TargetArrow.bInCombat:IsChecked()
	TargetArrow.Settings.bShowOutCombat = TargetArrow.bOutCombat:IsChecked()
	self:OnTargetUnitChanged(TargetArrow.currentUnit)
end

function TargetArrow:OnCloseButton()
	self.wndCfg:Show(false, true)
end

function TargetArrow:OnUnitOcclusionChanged(a,b,c,d,e,f)
	--Print(string.format("a: %s, b: %s, c: %s, d: %s, e: %s, f: %s", tostring(a), tostring(b), tostring(c), tostring(d), tostring(e), tostring(f)))
end

local TargetArrowInst = TargetArrow:new()
TargetArrowInst:Init()

STARTUP_FUNC("TargetArrow:End of TargetArrow.lua")